@@ -3,8 +3,8 @@
 
  Copyright (c) 1998 by Kevin Atkinson, (c) 1999-2001 by MySQL AB, and
  (c) 2004-2008 by Educational Technology Resources, Inc.  Others may
- also hold copyrights on code in this file.  See the CREDITS file in
- the top directory of the distribution for details.
+ also hold copyrights on code in this file.  See the CREDITS.txt file
+ in the top directory of the distribution for details.
 
  This file is part of MySQL++.
 
@@ -35,7 +35,7 @@ namespace mysqlpp {
 Query::Query(Connection* c, bool te, const char* qstr) :
 #if defined(MYSQLPP_HAVE_STD__NOINIT)
 // prevents a double-init memory leak in native VC++ RTL (not STLport!)
-std::ostream(std::_Noinit), 
+std::ostream(std::_Noinit),
 #else
 std::ostream(0),
 #endif
@@ -47,6 +47,7 @@ copacetic_(true)
 	init(&sbuffer_);
 	if (qstr) {
 		sbuffer_.str(qstr);
+		seekp(0, std::ios::end);	// allow more insertions at end
 	}
 }
 
@@ -83,7 +84,7 @@ Query::errnum() const
 }
 
 
-const char* 
+const char*
 Query::error() const
 {
 	return conn_->error();
@@ -215,7 +216,7 @@ Query::insert_id()
 }
 
 
-bool 
+bool
 Query::more_results()
 {
 	return conn_->driver()->more_results();
@@ -233,9 +234,9 @@ Query::operator=(const Query& rhs)
 	return *this;
 }
 
-Query::operator Query::private_bool_type() const
+Query::operator void*() const
 {
-	return *conn_ && copacetic_ ? &Query::copacetic_ : 0;
+	return *conn_ && copacetic_ ? const_cast<Query*>(this) : 0;
 }
 
 
@@ -247,7 +248,7 @@ Query::parse()
 	std::string name;
 
 	char* s = new char[sbuffer_.str().size() + 1];
-	memcpy(s, sbuffer_.str().data(), sbuffer_.str().size()); 
+	memcpy(s, sbuffer_.str().data(), sbuffer_.str().size());
 	s[sbuffer_.str().size()] = '\0';
 	const char* s0 = s;
 
@@ -455,7 +456,7 @@ Query::store(SQLQueryParms& p)
 }
 
 
-StoreQueryResult 
+StoreQueryResult
 Query::store(const SQLTypeAdapter& s)
 {
 	if ((parse_elems_.size() == 2) && !template_defaults.processing_) {
@@ -489,13 +490,13 @@ Query::store(const char* str, size_t len)
 		return StoreQueryResult(res, conn_->driver(), throw_exceptions());
 	}
 	else {
-		// Either result set is empty (which is copacetic), or there
-		// was a problem returning the result set (which is not).  If
-		// caller knows the result set will be empty (e.g. query is
-		// INSERT, DELETE...) it should call exec{ute}() instead, but
-		// there are good reasons for it to be unable to predict this.
-		copacetic_ = conn_->driver()->result_empty();
-		if (copacetic_) {
+		// Either result set is empty, or there was a problem executing
+		// the query or storing its results.  Since it's not an error to
+		// use store() with queries that never return results (INSERT,
+		// DELETE, CREATE, ALTER...) we need to figure out which case
+		// this is.  (You might use store() instead of execute() for
+		// such queries when the query strings come from "outside".)
+		if (copacetic_ = (conn_->errnum() == 0)) {
 			if (parse_elems_.size() == 0) {
 				// Not a template query, so auto-reset
 				reset();
@@ -523,7 +524,7 @@ Query::store_next()
 		if (res) {
 			return StoreQueryResult(res, conn_->driver(),
 					throw_exceptions());
-		} 
+		}
 		else {
 			// Result set is null, but throw an exception only i it is
 			// null because of some error.  If not, it's just an empty
@@ -531,25 +532,25 @@ Query::store_next()
 			// set if exceptions are disabled, as well.
 			if (conn_->errnum() && throw_exceptions()) {
 				throw BadQuery(error(), errnum());
-			} 
+			}
 			else {
 				return StoreQueryResult();
 			}
 		}
 	}
 	else if (throw_exceptions()) {
-        if (rc == DBDriver::nr_error) {
-            throw BadQuery(error(), errnum());
-        }
+		if (rc == DBDriver::nr_error) {
+			throw BadQuery(error(), errnum());
+		}
 		else if (conn_->errnum()) {
 			throw BadQuery(error(), errnum());
-        }
+		}
 		else {
 			return StoreQueryResult();	// normal end-of-result-sets case
 		}
-    }
-    else {
-        return StoreQueryResult();
+	}
+	else {
+		return StoreQueryResult();
 	}
 #else
 	return store();
@@ -610,13 +611,9 @@ Query::use(const char* str, size_t len)
 		return UseQueryResult(res, conn_->driver(), throw_exceptions());
 	}
 	else {
-		// Either result set is empty (which is copacetic), or there
-		// was a problem returning the result set (which is not).  If
-		// caller knows the result set will be empty (e.g. query is
-		// INSERT, DELETE...) it should call exec{ute}() instead, but
-		// there are good reasons for it to be unable to predict this.
-		copacetic_ = conn_->driver()->result_empty();
-		if (copacetic_) {
+		// See comments in store() above for why we distinguish between
+		// empty result sets and actual error returns here.
+		if (copacetic_ = (conn_->errnum() == 0)) {
 			if (parse_elems_.size() == 0) {
 				// Not a template query, so auto-reset
 				reset();
