@@ -4,8 +4,8 @@
 /***********************************************************************
  Copyright (c) 1998 by Kevin Atkinson, (c) 1999-2001 by MySQL AB, and
  (c) 2004-2008 by Educational Technology Resources, Inc.  Others may
- also hold copyrights on code in this file.  See the CREDITS file in
- the top directory of the distribution for details.
+ also hold copyrights on code in this file.  See the CREDITS.txt file
+ in the top directory of the distribution for details.
 
  This file is part of MySQL++.
 
@@ -115,16 +115,9 @@ class MYSQLPP_EXPORT Connection;
 /// See the user manual for more details about these options.
 
 class MYSQLPP_EXPORT Query :
-        public std::ostream,
+		public std::ostream,
 		public OptionalExceptions
 {
-private:
-	/// \brief Pointer to bool data member, for use by safe bool
-	/// conversion operator.
-	///
-	/// \see http://www.artima.com/cppsource/safebool.html
-    typedef bool Query::*private_bool_type;
-
 public:
 	/// \brief Create a new query object attached to a connection.
 	///
@@ -227,7 +220,11 @@ public:
 	///
 	/// \retval 0 if the previous query did not generate an ID.  Use
 	/// the SQL function LAST_INSERT_ID() if you need the last ID
-	/// generated by any query, not just the previous one.
+	/// generated by any query, not just the previous one.  This
+	/// applies to stored procedure calls because this function returns
+	/// the ID generated by the last query, which was a CALL statement,
+	/// and CALL doesn't generate IDs.  You need to use LAST_INSERT_ID()
+	/// to get the ID in this case.
 	ulonglong insert_id();
 
 	/// \brief Assign another query's state to this object
@@ -254,7 +251,7 @@ public:
 	/// This method returns false if either the Query object or its
 	/// associated Connection object has seen an error condition since
 	/// the last operation.
-	operator private_bool_type() const;
+	operator void*() const;
 
 	/// \brief Treat the contents of the query string as a template
 	/// query.
@@ -345,11 +342,11 @@ public:
 	SimpleResult execute() { return execute(str(template_defaults)); }
 
 	/// \brief Execute template query using given parameters.
-    ///
-    /// This method should only be used by code that doesn't know,
-    /// at compile time, how many parameters it will have.  This is
-    /// useful within the library, and also for code that builds
-    /// template queries dynamically, at run time.
+	///
+	/// This method should only be used by code that doesn't know,
+	/// at compile time, how many parameters it will have.  This is
+	/// useful within the library, and also for code that builds
+	/// template queries dynamically, at run time.
 	///
 	/// \param p parameters to use in the template query.
 	SimpleResult execute(SQLQueryParms& p);
@@ -407,11 +404,11 @@ public:
 
 	/// \brief Execute a template query that can return rows, with
 	/// access to the rows in sequence
-    ///
-    /// This method should only be used by code that doesn't know,
-    /// at compile time, how many parameters it will have.  This is
-    /// useful within the library, and also for code that builds
-    /// template queries dynamically, at run time.
+	///
+	/// This method should only be used by code that doesn't know,
+	/// at compile time, how many parameters it will have.  This is
+	/// useful within the library, and also for code that builds
+	/// template queries dynamically, at run time.
 	///
 	/// \param p parameters to use in the template query.
 	UseQueryResult use(SQLQueryParms& p);
@@ -470,11 +467,11 @@ public:
 	StoreQueryResult store() { return store(str(template_defaults)); }
 
 	/// \brief Store results from a template query using given parameters.
-    ///
-    /// This method should only be used by code that doesn't know,
-    /// at compile time, how many parameters it will have.  This is
-    /// useful within the library, and also for code that builds
-    /// template queries dynamically, at run time.
+	///
+	/// This method should only be used by code that doesn't know,
+	/// at compile time, how many parameters it will have.  This is
+	/// useful within the library, and also for code that builds
+	/// template queries dynamically, at run time.
 	///
 	/// \param p parameters to use in the template query.
 	StoreQueryResult store(SQLQueryParms& p);
@@ -753,7 +750,8 @@ public:
 			MYSQL_ROW d = result.fetch_raw_row();
 			if (!d)
 				break;
-			Row row(d, &result, result.fetch_lengths(), true);
+			Row row(d, &result, result.fetch_lengths(),
+					throw_exceptions());
 			if (!row)
 				break;
 			con.push_back(typename Sequence::value_type(row));
@@ -761,20 +759,20 @@ public:
 	}
 
 	/// \brief Execute template query using given parameters, storing
-    /// the results in a sequence type container.
-    ///
-    /// This method should only be used by code that doesn't know,
-    /// at compile time, how many parameters it will have.  This is
-    /// useful within the library, and also for code that builds
-    /// template queries dynamically, at run time.
+	/// the results in a sequence type container.
+	///
+	/// This method should only be used by code that doesn't know,
+	/// at compile time, how many parameters it will have.  This is
+	/// useful within the library, and also for code that builds
+	/// template queries dynamically, at run time.
 	///
 	/// \param con container that will receive the results
 	/// \param p parameters to use in the template query.
-    template <class Seq>
-    void storein_sequence(Seq& con, SQLQueryParms& p)
-    {
-        storein_sequence(con, str(p));
-    }
+	template <class Seq>
+	void storein_sequence(Seq& con, SQLQueryParms& p)
+	{
+		storein_sequence(con, str(p));
+	}
 
 	/// \brief Execute a query, storing the result set in an STL
 	/// associative container.
@@ -810,7 +808,8 @@ public:
 			MYSQL_ROW d = result.fetch_raw_row();
 			if (!d)
 				return;
-			Row row(d, &result, result.fetch_lengths(), true);
+			Row row(d, &result, result.fetch_lengths(),
+					throw_exceptions());
 			if (!row)
 				break;
 			con.insert(typename Set::value_type(row));
@@ -818,20 +817,20 @@ public:
 	}
 
 	/// \brief Execute template query using given parameters, storing
-    /// the results in a set type container.
-    ///
-    /// This method should only be used by code that doesn't know,
-    /// at compile time, how many parameters it will have.  This is
-    /// useful within the library, and also for code that builds
-    /// template queries dynamically, at run time.
+	/// the results in a set type container.
+	///
+	/// This method should only be used by code that doesn't know,
+	/// at compile time, how many parameters it will have.  This is
+	/// useful within the library, and also for code that builds
+	/// template queries dynamically, at run time.
 	///
 	/// \param con container that will receive the results
 	/// \param p parameters to use in the template query.
-    template <class Set>
-    void storein_set(Set& con, SQLQueryParms& p)
-    {
-        storein_set(con, str(p));
-    }
+	template <class Set>
+	void storein_set(Set& con, SQLQueryParms& p)
+	{
+		storein_set(con, str(p));
+	}
 
 	/// \brief Execute a query, and store the entire result set
 	/// in an STL container.
